package com.perfectech.tis.activities;

import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.R;
import com.perfectech.tis.syncClasses.SyncTasks;
import com.perfectech.tis.TISDBHelper;

import java.io.IOException;
import java.util.ArrayList;

public class DVIRCompleteReportActivity extends AppCompatActivity {
    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 6;
    private static String sUserID = "";
    private static ArrayList<String> reportArray;
    private static ArrayList<ArrayList<String>> reportItemsArray;
    private boolean bcompletedDVIR = false;
    // These are the only fields that can be updated
    EditText etEndMiles;
    EditText etOOTMiles;
    TextView tvBeginMiles;
    private static String sDVIRNo;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dvircomplete_report);
        etEndMiles = (EditText) findViewById(R.id.etDVIRCREM);
        etOOTMiles = (EditText) findViewById(R.id.etDVIRCROOTMi);
        tvBeginMiles = (TextView) findViewById(R.id.tvDVIRCRBeginMiles);
        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        TextView Header = (TextView) findViewById(R.id.tvDVIRCRUserName);
        Header.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));
        String sCurrentVersion = prefs.getString("TISVersion", "No Value");
        TextView otvCopyright = (TextView) findViewById(R.id.tvCopyright);
        String sVersionText = otvCopyright.getText().toString();
        // need to replace the %RELNUM% with the actual rel version
        sVersionText = sVersionText.replace("RELNUM", sCurrentVersion);
        otvCopyright.setText(sVersionText);
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            sDVIRNo = extras.getString("DVIRNo");
            if (!getReportFromDB(sDVIRNo)) {
                // These was an error populating the data. We need to bail to DVIR reports
                String sComments = "Something is wrong with the items in the database for this DVIR";
                String sTitle = "Something Is Wrong";
                errorDialogue(sTitle, sComments);
                Intent iReports = new Intent(this, DVIRReportsActivity.class);
                startActivity(iReports);
            }
        } else {
            //nothing passed from intent. This is a failure and we need to alert and bail
            String sComments = "Something is wrong with the items in the database for this DVIR";
            String sTitle = "Something Is Wrong";
            errorDialogue(sTitle, sComments);
            Intent iReports = new Intent(this, DVIRReportsActivity.class);
            startActivity(iReports);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_dvircomplete_report, menu);
        if (bcompletedDVIR) {
            MenuItem saveMenuItem = menu.findItem(R.id.action_save);
            saveMenuItem.setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_save) {
            if (saveReport(sDVIRNo)) {
                queueForSync();
                Intent i = new Intent(getApplicationContext(), DVIRReportsActivity.class);
                startActivity(i);
                finish();
            }
            return true;
        }
        if (id == R.id.action_cancel) {
            if (saveReport(sDVIRNo)) {
                Intent i = new Intent(getApplicationContext(), DVIRReportsActivity.class);
                startActivity(i);
                finish();
            }
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "DVIRCOmpleteReportsActivity");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /*
    Get the values from the database to populate the form.  These values are from when the
    form was initially filled out.
     */
    private boolean getReportFromDB (String sReportnum) {
        Boolean bRtnValue = false;
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();

        String sSQL = "select CompletedDVIR, Remarks, Location, BeginMileage, TruckNo, " +
                "TrailerNo, DVIRNo,  EndMileage, OutOfStateMileage " +
                "from DVIREntered where DVIRNo = '" + sReportnum + "'";
        Log.d("sSQL", sSQL);

        reportArray = oDB.getList(sSQL, null);

        sSQL = "select DVIRFields.Title, DVIRFieldValues.Value ";
        sSQL += "from DVIRFields, DVIRFieldValues where DVIRFields._id = DVIRFieldValues.FieldID ";
        sSQL += "and DVIRFieldValues.DVIRNo = '" + sReportnum + "'";
        Log.d("sSQL", sSQL);

        reportItemsArray = oDB.selectRecordsFromDBList(sSQL, null);

        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
            bRtnValue = false;
        }

        if (reportArray.size() > 0) {
            try {
                // Update the top of the view
                TextView TruckNo = (TextView) findViewById(R.id.tvDVIRCRTruckNo);
                TextView TrailerNo = (TextView) findViewById(R.id.tvDVIRCRTrailerNo);
                TextView BeginMiles = (TextView) findViewById(R.id.tvDVIRCRBeginMiles);
                TextView Location = (TextView) findViewById(R.id.tvDVIRCRLoc);
                TextView Remarks = (TextView) findViewById(R.id.tvDVIRCRRemarks);

                TruckNo.setText(reportArray.get(4));
                TrailerNo.setText(reportArray.get(3));
                BeginMiles.setText(reportArray.get(5));
                Location.setText(reportArray.get(6));
                Remarks.setText(reportArray.get(7));
                if (reportArray.get(8).toUpperCase().equals("YES")) {
                    etEndMiles.setHint("");
                    etOOTMiles.setHint("");
                    etEndMiles.setEnabled(false);
                    etOOTMiles.setEnabled(false);
                    etEndMiles.setText(reportArray.get(1));
                    etOOTMiles.setText(reportArray.get(0));
                    bcompletedDVIR = true;
                } else {
                    //Display the keyboard and focus the end miles field
                    etEndMiles.requestFocus();
                    getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
                }
                bRtnValue = true;
            } catch (Exception ex) {
                bRtnValue = false;
            }
        } else {
            // We did not get anything from the database. This should not happen.
            bRtnValue = false;
        }

        if (reportItemsArray.size() == 0) {
            //something went wrong there should always be items
            bRtnValue = false;
        }
        if (bRtnValue) {
            populateBottomOfView();
        }

        return bRtnValue;
    }

    /*
    Populate the items that were checked when the initial DVIR was created
     */
    private void populateBottomOfView() {
        ScrollView svCheckItems = (ScrollView) findViewById(R.id.svDVIRCRItems);
        svCheckItems.setScrollbarFadingEnabled(false);
        TableLayout tlCheckListItems = (TableLayout) findViewById(R.id.tlDVIRCRItems);
        //Array size was already tested so no need to test here
        for (ArrayList<String> sValue : reportItemsArray) {
            TableRow trItems = new TableRow(this);
            View v = LayoutInflater.from(this).inflate(R.layout.dvir_rep_comp_row, trItems, false);
            TextView tvItem = (TextView) v.findViewById(R.id.tvDVIRCRItem);
            tvItem.setText(sValue.get(0));
            TextView tvAnswer = (TextView) v.findViewById(R.id.tvDVIRCRItemAnswer);
            tvAnswer.setText(sValue.get(1));
            tlCheckListItems.addView(v);
        }
    }

    private boolean validateInput () {
        boolean bRtnValue = true;
        // There must be something in the end miles field
        if (etEndMiles.getText().length() < 1) {
            //Pull up a dialogue box
            String sComments = "The End Miles (End Mi) field must be filled in.";
            String sTitle = "Not Complete";
            errorDialogue(sTitle, sComments);
            bRtnValue = false;
        }
        if (bRtnValue) {
            // Make sure that end mileage is greater than the begin mileage
            Integer iBeginMiles = Integer.parseInt(tvBeginMiles.getText().toString());
            Integer iEndMiles = Integer.parseInt(etEndMiles.getText().toString());
            if (iEndMiles < iBeginMiles) {
                bRtnValue = false;
                String sComments = "The End Miles are less than the Begin Miles.  Please check your mileage.";
                String sTitle = "Mileage Error";
                errorDialogue(sTitle, sComments);
            }
        }
        return bRtnValue;
    }

    private boolean saveReport(String sDVIRNo) {
        Boolean bRtnValue = true;
        bRtnValue = validateInput();
        if (bRtnValue) {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                bRtnValue = false;
                DBLogs.insert("Error: " + ioe.toString(),
                        "DVIREntryActivity.saveItems", getApplicationContext(),
                        sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();
            String sEndMiles = etEndMiles.getText().toString();
            String sOOTMiles = etOOTMiles.getText().toString();
            ContentValues oValues = new ContentValues();
            oValues.put("CompletedDVIR", "YES");
            oValues.put("EndMileage", sEndMiles);
            if (sOOTMiles.length() > 0) {
                oValues.put("OutOfStateMileage", sOOTMiles);
            }
            String sWhere = "DVIRNo = '" + sDVIRNo + "'";
            oDB.updateRecordsInDB("DVIREntered", sWhere, oValues);
            oDB.close();
        }
        return bRtnValue;
    }

    private void queueForSync() {
        // queue the message to be sent
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sCurrentDateTime = oDateConvertClass.GMTTimestamp();
        SyncTasks syncTasks = new SyncTasks();
        syncTasks.oContext = getApplicationContext();
        syncTasks.sRecType = "DVIRC";
        syncTasks.sRecID = sDVIRNo;
        syncTasks.sDateTime = sCurrentDateTime;
        syncTasks.sQueued = "0";
        syncTasks.sDateTimeQueued = "";
        syncTasks.sCompleted = "0";
        syncTasks.sDateTimeCompleted = "";
        syncTasks.write();
    }

    private void errorDialogue(String sTitle, String sComment) {
        TextView title = new TextView(this);
        title.setText(sTitle);
        title.setBackgroundColor(Color.RED);
        title.setPadding(10, 10, 10, 10);
        title.setGravity(Gravity.CENTER);
        title.setTextColor(Color.WHITE);
        title.setTextSize(20);
        new AlertDialog.Builder(this)
                .setCustomTitle(title)
                .setMessage(sComment)
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // Do nothing but close the dialog
                        dialog.dismiss();
                    }
                })
                .show();
    }
}
