package com.perfectech.tis.activities;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ContentValues;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.ProgressDialog;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.database.SQLException;
import android.graphics.Color;
import android.graphics.Paint;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.StrictMode;
import android.provider.Settings;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.TextView;
import android.util.Log;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DBParameters;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.R;
import com.perfectech.tis.syncClasses.TISCommon;
import com.perfectech.tis.TISDBHelper;

public class TISActivity extends Activity {

    /**
     * ****************************************************************************
     */
    /* Define Global Variables                                   */
    /*
     * ****************************************************************************
     */
    private static final String sDBName = "TIS.db";
    private static String[] userInfoArray;
    private static String[] deviceInfoArray;
    static String sDeviceID;
    static int iVersionVal = 6;
    private String sAndroidID;

    // Current version variables
    private String sCurrentVersion;
    private String sSoftwareName;
    private String sSoftwareUpdateVersionCheck;
    private String sSoftwareUpdateURL;
    private String sRestBaseURL;
    private String sLoggedInUserCompany = "";
    private String sUserID = "";
    private static boolean debug_mode = false;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        DBParameters oDBParameters = new DBParameters();
        try {
            String sDebugMode = oDBParameters.readParameter("DebugMode",
                    getApplicationContext(), sDBName, iVersionVal);
            if (sDebugMode.equals("1")) {
                debug_mode = true;
            } else {
                debug_mode = false;
            }
            oDBParameters.updateParameter("DeviceID", sDeviceID,
                    getApplicationContext(), sDBName, iVersionVal);
        } catch(Exception ex) {
            // To do
        }

        StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder()
                .permitAll().build();
        StrictMode.setThreadPolicy(policy);

        // need to setup shared preferences so that OMSAutoStart application
        // can read the current version of this application
        updateLocalSharedPreferences();

        sAndroidID = Settings.System.getString(getContentResolver(),
                Settings.System.ANDROID_ID);

        // Log.d("sDeviceID: ", sDeviceID);
        Log.d("AndroidID: ", sAndroidID);

        // need to get a unique device ID if one is not in already defined
        if (!checkDevice()) {
            if (TISCommon.isOnline()) {

                getDeviceID();
            }
        }

        if (debug_mode == true) {
            DBLogs.insert("Application is starting up", "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("AndroidID: " + sAndroidID, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("DeviceID: " + sDeviceID, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("Device Manufacturer: " + android.os.Build.MANUFACTURER, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("Device Model: " + android.os.Build.MODEL, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("Device Serial: " + android.os.Build.SERIAL, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            DBLogs.insert("Device OS Version: " + android.os.Build.VERSION.RELEASE, "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
        }

        // show the login screen
        setContentView(R.layout.login);
        getRememberMe();

        final Button loginBtn = (Button) findViewById(R.id.btnLogin);
        final EditText UserNameTxt = (EditText) findViewById(R.id.userName);
        final EditText PasswordTxt = (EditText) findViewById(R.id.password);
        final TextView ErrorTv = (TextView) findViewById(R.id.tvError);
        final Button btnforgotPw = (Button) findViewById(R.id.btnForgotPW);
        final TextView tvLink = (TextView) findViewById(R.id.tvLoginRegText);
        tvLink.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);

        //Add userID if it is not blank
        if (sUserID.length() > 0) {
            UserNameTxt.setText(sUserID);
            PasswordTxt.requestFocus();
        }
        btnforgotPw.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                SharedPreferences tisLoginPrefs = getSharedPreferences("TISlogin",
                        Context.MODE_PRIVATE);
                //Make sure to set remember me to false.
                SharedPreferences.Editor editor = tisLoginPrefs.edit();
                editor.putBoolean("remember_me", false);
                editor.apply();
                EditText PasswordTxt = (EditText) findViewById(R.id.password);
                PasswordTxt.setText("");
                CheckBox cbRememberme = (CheckBox) findViewById(R.id.rememberMe);
                cbRememberme.setChecked(false);
                Intent i = new Intent(getApplicationContext(), ForgotPassword.class);
                startActivity(i);
            }
        });

        tvLink.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                doRegistrationLink();
            }
        });


        loginBtn.setOnClickListener(new OnClickListener() {

            public void onClick(View v) {
                // Validate the user
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(UserNameTxt.getWindowToken(), 0);
                imm.hideSoftInputFromWindow(PasswordTxt.getWindowToken(), 0);

                // It is important for the userid to be lower case or it really messes with
                // the database lookups
                sUserID = UserNameTxt.getText().toString().toLowerCase();
                String sPassword = PasswordTxt.getText().toString();
                Log.d("sUserName: ", sUserID);
                Log.d("sPassword: ", sPassword);

                if (sUserID.length() > 0 && sPassword.length() > 0) {
                    // let's validate the user
                    switch (doLogin(sUserID, sPassword)) {
                        case 0:
                            ErrorTv.setText("Invalid username or password.");
                            break;
                        case 1:
                            displayAcceptance(sUserID);
                            break;
                        case 2:
                            ErrorTv.setText("Not paid for services.");
                            break;
                        default:
                            //Should never get here
                            ErrorTv.setText("Unknown error.");
                            break;
                    }
                } else {
                    ErrorTv.setText("User name and password is required.");
                }
            }
        });

        TextView otvSupportNumber = (TextView) findViewById(R.id.tvLoginHelpText);
        otvSupportNumber.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                dialPhone("8887748044");
            }
        });

        // Clear the error text view every time a key is pressed to remove
        // the login error if it exists
        UserNameTxt.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                ErrorTv.setText("");
                return false;
            }
        });

        // Clear the error text view every time a key is pressed to remove
        // the login error if it exists
        PasswordTxt.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                ErrorTv.setText("");
                return false;
            }
        });

        //Test for location services being turned on
        testLocationServices();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        Log.d("TISActivity.onRestart()", "onRestart() method started");
        getDBSharedPreferancesData();
        updateLocalSharedPreferences();
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.d("TISActivity.onResume()", "onResume() method started");
        getDBSharedPreferancesData();
        updateLocalSharedPreferences();
    }

    /**
     * *************************************************************************************
     */
    /* This method will validate the user logging in                                         */
    /* ***************************************************************************************/
    private Integer doLogin(String sUserName, String sPassword) {
        // Return value of 0 mean nothing returned, 1 means returned and paid 2 means return and not paid
        Integer iRtnVal = 0;

        // check the database for the user
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            DBLogs.insert("Error: " + ioe.toString(), "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);

            throw new Error("Unable to create database");
        }

        try {
            oDB.openDataBase();
        } catch (SQLException sqe) {
            DBLogs.insert("Error: " + sqe.toString(), "TISActivity",
                    getApplicationContext(), sDBName, iVersionVal);
            iRtnVal = 0;
        }

        //First check to see if EmployeeType equals user_type_id for any user since this means
        //that this employee has logged in before
        String sSQL = "select * from EmployeeMaster where UserID = '" + sUserName.toLowerCase() + "'";
        ArrayList<ArrayList<String>> sStringList = oDB.selectRecordsFromDBList(
                sSQL, null);
        oDB.close();
        switch (sStringList.size()) {
            case 0:
                //This means the userid does not exist
                iRtnVal = checkServerForLogin(sUserName, sPassword, "NEW");
                break;
            case 1:
                ArrayList<String> tempList = sStringList.get(0);
                //Must go to the server if the internal database shows not paid
                if (tempList.get(0).equals("0")) {
                    iRtnVal = checkServerForLogin(sUserName, sPassword, "UPDATE");
                } else {
                    iRtnVal = validateUserNameandPassword(tempList, sUserName, sPassword);
                }
                break;
            default:
                //There is something wrong in the database if there is more than one user
                //with EmployeeType of user_type_id
                iRtnVal = 0;
                TextView title = new TextView(this);
                title.setText("Wrong User");
                title.setBackgroundColor(Color.RED);
                title.setPadding(10, 10, 10, 10);
                title.setGravity(Gravity.CENTER);
                title.setTextColor(Color.WHITE);
                title.setTextSize(20);
                new AlertDialog.Builder(this)
                        .setCustomTitle(title)
                        .setMessage("There is something wrong in the database. Please contact support.")
                        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                // Do nothing but close the dialog
                                dialog.dismiss();
                            }
                        })
                        .show();
                break;
        }
        if (iRtnVal == 1) {
            //update login preferences
            setRememberMe();
            // need to get the name and company from the array
            for (int i = 0; i < sStringList.size(); i++) {
                int iSize = sStringList.size();
                ArrayList<String> tempList = sStringList.get(i);
                String sFirstName = tempList.get(6);
                String sLastName = tempList.get(7);
                String sCompanyName = tempList.get(9);
                sLoggedInUserCompany = sFirstName + " " + sLastName + " - " + sCompanyName;
            }
            sUserID = sUserName.toLowerCase();
            updateLocalSharedPreferences();
            DBParameters oDBParameters = new DBParameters();
            oDBParameters.deleteParameter("CurrentUserID",
                    getApplicationContext(), sDBName, iVersionVal);
            oDBParameters.insertParameter("CurrentUserID", sUserName,
                    getApplicationContext(), sDBName, iVersionVal);
        }
        return iRtnVal;
    }

    //Validate the user name and password
    private Integer validateUserNameandPassword(ArrayList<String> tempList, String sUserName, String sPassword) {
        // Return value of 0 mean nothing returned, 1 means returned and paid 2 means return and not paid
        Integer iRtnValue = 0;
        //check to see if userid and the password match
        String sDBPassword = tempList.get(3);
        String sDBUserName = tempList.get(4);
        if (sDBUserName.toLowerCase().equals(sUserName)) {
            //Now see if password matches
            if (sDBPassword.equals(sPassword)) {
                iRtnValue = 1;
            }
            if (tempList.get(0).equals("0")) {
                //This means not paid
                iRtnValue = 2;
            }
        } else {
            //if userid does not match the one in the database show error
            TextView title = new TextView(this);
            title.setText("Wrong User");
            title.setBackgroundColor(Color.RED);
            title.setPadding(10, 10, 10, 10);
            title.setGravity(Gravity.CENTER);
            title.setTextColor(Color.WHITE);
            title.setTextSize(20);
            new AlertDialog.Builder(this)
                    .setCustomTitle(title)
                    .setMessage("User " + sUserName + " does not match user in database. There is only one user allowed per device.")
                    .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            // Do nothing but close the dialog
                            dialog.dismiss();
                        }
                    })
                    .show();
        }
        return iRtnValue;
    }

    /*****************************************************************************************/
    /* This method will display the acceptance dialog                                        */

    /**
     * *************************************************************************************
     */
    private Integer checkServerForLogin(String sUserName, String sPassword, String sType) {
        // Return value of 0 mean nothing returned, 1 means returned and paid 2 means return and not paid
        Integer iRtnVal = 0;
        DBParameters oDBParameters = new DBParameters();

        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", getApplicationContext(), sDBName,
                iVersionVal); //"http://androidexample.com/media/webservice/JsonReturn.php";

        serverURL += "doLogin.php";

        String data = "";

        try {
            // Set Request parameter
            data += "&" + URLEncoder.encode("userName", "UTF-8") + "=" + sUserName;
            data += "&" + URLEncoder.encode("password", "UTF-8") + "=" + sPassword;
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        /************ Make Post Call To Web Server ***********/
        BufferedReader reader = null;
        StringBuilder sb = new StringBuilder();

        // Send data
        try {

            // Defined URL  where to send data
            URL url = new URL(serverURL);

            // Send POST data request
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(data);
            wr.flush();

            // Get the server response
            reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String line = null;

            // Read Server Response
            while ((line = reader.readLine()) != null) {
                // Append server response in string
                sb.append(line + "");

            }
            //Checking lenght of returned data and if bigger that 10 characters the network
            //login was successfull
            if (sb.length()>10) {
                iRtnVal = 1;
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            try {
                reader.close();
            } catch (Exception ex) {
            }
        }


        // put the information into the database
        if (iRtnVal == 1) {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName, null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            try {
                JSONArray oJsonArray = new JSONArray(sb.toString());

                for (int iCounter = 0; iCounter < oJsonArray.length(); iCounter++) {

                    JSONObject json = oJsonArray.getJSONObject(iCounter);
                    ContentValues oValues = new ContentValues();
                    oValues.put("Paid", json.getString("paid"));
                    oValues.put("UserID", json.getString("username").toLowerCase());
                    oValues.put("Password", json.getString("password"));
                    oValues.put("FirstName", json.getString("first_name"));
                    oValues.put("LastName", json.getString("last_name"));
                    oValues.put("CompanyName", json.getString("company_name"));
                    oValues.put("EmailAddress", json.getString("email_address"));
                    //oValues.put("EmployeeType", ""); //Not needed for this app
                    oValues.put("Title", "title");

                    sLoggedInUserCompany = json.getString("first_name") + " " + json.getString("last_name") + " - " + json.getString("company_name");

                    try {
                        if (sType.equals("NEW")) {
                            oDB.insertRecordsInDB("EmployeeMaster", null, oValues);
                        } else {
                            oDB.updateRecordsInDB("EmployeeMaster", "UserID = " + oValues.getAsString("UserID"), oValues);
                        }
                    } catch (Exception sqlEx) {
                        Log.e("sqlEx:", sqlEx.toString());
                    }
                    if (oValues.getAsInteger("Paid") == 0) {
                        iRtnVal = 2;
                    } else {
                        iRtnVal = 1;
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                oDB.close();
            }
        }


        // Use AsyncTask execute Method To Prevent ANR Problem
        //new LongOperation().execute(serverURL);
        if (iRtnVal != 0) {
            sUserID = sUserName.toLowerCase();
            updateLocalSharedPreferences();
        }

        return iRtnVal;
    }
    /*****************************************************************************************/
    /* This method will display the acceptance dialog */

    /**
     * *************************************************************************************
     */
    private void displayAcceptance(String sUserName) {

        // see if the driver has already accepted the policy
        boolean bAlreadyAccepted = false;
        bAlreadyAccepted = checkUser(sUserName);
        updateLocalSharedPreferences();

        if (bAlreadyAccepted) {
            finish();
            Intent i = new Intent(getApplicationContext(), StartupActivity.class);
            i.putExtra("UserID", sUserName);
            startActivity(i);
        } else {
            Intent i = new Intent(getApplicationContext(), UserPolicy.class);
            i.putExtra("UserID", sUserName);
            startActivity(i);
            finish();
        }
    }

    /*****************************************************************************************/
    /* This method will display the acceptance dialog */

    /**
     * *************************************************************************************
     */
    private boolean checkUser(String sUserName) {
        boolean bRtnVal = false;
        try {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "TISActivity.checkUser:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            // select the ticket information from the DB
            String sSQL = "select AcceptedUsagePolicy ";
            sSQL += "from EmployeeMaster ";
            sSQL += "where UserID =  '" + sUserName + "' ";

            ArrayList<String> userArray = oDB.getList(sSQL, null);

            // closes the database
            try {
                oDB.close();
            } catch (Throwable e) {
                DBLogs.insert("Error: " + e.toString(),
                        "TISActivity.checkUser:closeDB",
                        getApplicationContext(), sDBName, iVersionVal);
                e.printStackTrace();
            }

            userInfoArray = new String[userArray.size()];
            userInfoArray = userArray.toArray(userInfoArray);

            if (userInfoArray[0].equals("1")) {
                bRtnVal = true;
            } else {
                bRtnVal = false;
            }
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(), "TISActivity.checkUser",
                    getApplicationContext(), sDBName, iVersionVal);
            bRtnVal = false;
        }

        return bRtnVal;
    }

    /********************************************************************************/
	/* This method will get a unique device number for this device */

    /**
     * ****************************************************************************
     */
    private boolean getDeviceID() {
        boolean bRtnVal = false;

        if (debug_mode == true) {
            DBLogs.insert("INFO: getting unique deviceID",
                    "TISActivity.getDeviceID", getApplicationContext(), sDBName,
                    iVersionVal);
        }
        /*
		getDeviceID oGetDeviceID = new getDeviceID();
		oGetDeviceID.oContext = getApplicationContext();
		try {
			oGetDeviceID.getUniqueDeviceID(sAndroidID);
			bRtnVal = checkDevice();
		} catch (Exception e) {
			Log.d("OMSActivity.getDeviceID Error", e.toString());
			bRtnVal = false;
		}
		*/
        return bRtnVal;
    }

    /*****************************************************************************************/
	/* This method will display the acceptance dialog */

    /**
     * *************************************************************************************
     */
    private boolean checkDevice() {
        boolean bRtnVal = false;
        try {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "TISActivity.checkDevice:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            // select the ticket information from the DB
            String sSQL = "select DeviceID ";
            sSQL += "from DeviceMaster ";
            sSQL += "where AndroidID =  '" + sAndroidID + "' ";

            ArrayList<String> deviceArray = oDB.getList(sSQL, null);

            // closes the database
            try {
                oDB.close();
            } catch (Throwable e) {
                DBLogs.insert("Error: " + e.toString(),
                        "TISActivity.checkDevice:closeDB",
                        getApplicationContext(), sDBName, iVersionVal);
                e.printStackTrace();
            }

            deviceInfoArray = new String[deviceArray.size()];
            deviceInfoArray = deviceArray.toArray(deviceInfoArray);

            if (deviceArray.size() > 0) {
                sDeviceID = deviceInfoArray[0];
                bRtnVal = true;
            } else {
                bRtnVal = false;
            }
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(), "TISActivity.checkDevice",
                    getApplicationContext(), sDBName, iVersionVal);
            bRtnVal = false;
        }

        return bRtnVal;
    }

    private void testLocationServices() {
        //Test if location services are turned on and if not prompt the user to turn them on
        LocationManager lm = (LocationManager) getSystemService(LOCATION_SERVICE);
        if(!lm.isProviderEnabled(LocationManager.GPS_PROVIDER) &&
                !lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
            // Build the alert dialog
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle(R.string.gps_not_found_title);
            builder.setMessage(R.string.gps_not_found_message);
            builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialogInterface, int i) {
                    // Show location settings when the user acknowledges the alert dialog
                    Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                    startActivity(intent);
                }
            });
            Dialog alertDialog = builder.create();
            alertDialog.setCanceledOnTouchOutside(false);
            alertDialog.show();
        }
    }

    /*****************************************************************************************/
	/* This method will get the current version of the installed application */

    /**
     * *************************************************************************************
     */
    public String getVersion(Context context) {
        try {
            PackageInfo pInfo = context.getPackageManager().getPackageInfo(
                    context.getPackageName(), PackageManager.GET_META_DATA);
            return pInfo.versionName.toString();
        } catch (NameNotFoundException e) {
            return "";
        }
    }

    /*****************************************************************************************/
	/* This method will get all values needed for SharedPreferences */

    /**
     * *************************************************************************************
     */
    private void getDBSharedPreferancesData() {
        DBParameters oDBParameters = new DBParameters();
        sSoftwareName = oDBParameters.readParameter("SoftwareName",
                getApplicationContext(), sDBName, iVersionVal);
        sSoftwareUpdateVersionCheck = oDBParameters.readParameter(
                "SoftwareUpdateVersionCheck", getApplicationContext(), sDBName,
                iVersionVal);
        sSoftwareUpdateURL = oDBParameters.readParameter("SoftwareUpdateURL",
                getApplicationContext(), sDBName, iVersionVal);
        sCurrentVersion = getVersion(this);
        sRestBaseURL = oDBParameters.readParameter("REST_BASE_URL",
                getApplicationContext(), sDBName, iVersionVal);
        sUserID = oDBParameters.readParameter("CurrentUserID",
                getApplicationContext(), sDBName, iVersionVal);
        sUserID = sUserID.trim();
    }

    /*****************************************************************************************/
	/* This method will update the SharedPreferences */

    /**
     * *************************************************************************************
     */
    private void updateLocalSharedPreferences() {

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        sCurrentVersion = prefs.getString("TISVersion", "No Value");
        Log.d("updateSharedPrefereneces", sCurrentVersion);

        clearLocalSharedPreferences();
        getDBSharedPreferancesData();
        Log.i("sCurrentVersion", sCurrentVersion);
        Log.i("sSoftwareName", sSoftwareName);
        Log.i("sSoftwareUpdVerCheck", sSoftwareUpdateVersionCheck);
        Log.i("sSoftwareUpdateURL", sSoftwareUpdateURL);
        Log.i("sRestBaseURL", sRestBaseURL);
        editor.putString("TISVersion", sCurrentVersion).apply();
        editor.putString("SoftwareName", sSoftwareName).apply();
        editor.putString("SoftwareUpdateVersionCheck",
                sSoftwareUpdateVersionCheck).apply();
        editor.putString("SoftwareUpdateURL", sSoftwareUpdateURL).apply();
        editor.putString("RestBaseURL", sRestBaseURL).apply();
        editor.putString("LoggedInUserCompany", sLoggedInUserCompany).apply();
        editor.putString("UserID", sUserID.toLowerCase()).apply();
        editor.apply();

        SharedPreferences lastReminderCheck = getApplicationContext().getSharedPreferences("lastReminderCheck", Context.MODE_PRIVATE);
        SharedPreferences.Editor oEditor = lastReminderCheck.edit();
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sGMTNow = oDateConvertClass.GMTTimestamp();
        oEditor.putString("LastReminderCheck", sGMTNow);
        oEditor.commit();

    }

    /*****************************************************************************************/
	/* This method will clear the SharedPreferences */

    /**
     * *************************************************************************************
     */
    public void clearLocalSharedPreferences() {

        SharedPreferences sharedPreferences = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.clear();
        editor.apply();
    }

    private void getRememberMe() {
        CheckBox cbRememberme = (CheckBox) findViewById(R.id.rememberMe);
        EditText etPassword = (EditText) findViewById(R.id.password);
        try {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "TISActivity.checkUser:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            // select Remember Me from database and popuplate password if remember me set
            String sSQL = "select Password, RememberMe ";
            sSQL += "from EmployeeMaster ";
            sSQL += "where UserID =  '" + sUserID + "' ";

            ArrayList<String> userArray = oDB.getList(sSQL, null);
            oDB.close();
            if (userArray.get(0).equals("1")) {
                cbRememberme.setChecked(true);
                etPassword.setText(userArray.get(1));
            } else {
                cbRememberme.setChecked(false);
                etPassword.setText("");
            }
        } catch(Exception ex) {
            //todo
        }
    }

    private void setRememberMe() {
        CheckBox cbRememberme = (CheckBox) findViewById(R.id.rememberMe);
        try {
            TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "TISActivity.checkUser:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            // Update Remember me
            ContentValues oValues = new ContentValues();
            if (cbRememberme.isChecked()) {
                oValues.put("RememberMe", "1");
            } else {
                oValues.put("RememberMe", "0");
            }
            oDB.updateRecordsInDB("EmployeeMaster", "UserID = '" + sUserID + "'", oValues);
            oDB.close();
        } catch(Exception ex) {
            //todo
        }
    }

    private void dialPhone(String sPhoneNumber) {
        Intent callIntent = new Intent(Intent.ACTION_CALL);
        callIntent.setData(Uri.parse("tel:" + sPhoneNumber));
        //textView.getText().toString().trim()));
        startActivity(callIntent );
    }

    private void doRegistrationLink() {
        Intent newIntent = new Intent(Intent.ACTION_VIEW,
                Uri.parse("http://www.transportationis.com/register.php"));
        startActivity(newIntent);
    }
}